#!/System/Library/Frameworks/Ruby.framework/Versions/1.8/usr/bin/ruby

require 'osx/cocoa'
include OSX

libraryDirectoryPath = File.dirname(__FILE__)
$LOAD_PATH << libraryDirectoryPath

require 'lproj.rb'

def usage
	print "Usage:\n"
	print "-imageProjPath: path to an imageProj directory whose contents should be verified [required]\n"
	print "\n"
	print "Example:\n"
	print "ruby imageProj_check.rb\n -imageProjPath ~/Projects/Products/Release\\ -\\ Standard/Courier.app/Contents/Resources/snow_leopard.imageProj\n"
	print "\n"
	exit 1
end

if ARGV.count == 0
	usage
end

def start
	print "=========  IMAGEPROJ VALIDATION STARTED   ==========\n"
	print "\n"
end

def finish
	print "\n"
	print "=========  IMAGEPROJ VALIDATION SUCCESS   ==========\n"
	print "\n"
	exit 0
end

def fail(reason = nil, severity = :warning)
	print "\n"
	print "=========  IMAGEPROJ VALIDATION FAILURE   ==========\n"
	unless reason.nil?
		print "error: #{reason}\n"
	end
	print "\n"
	
	if severity == :fatal
		exit 1
	end
	
	def finish
		fail nil, :fatal
	end
end


# Check arguments

imageProjectFolder = NSUserDefaults.standardUserDefaults.stringForKey "imageProjPath"
if imageProjectFolder.nil?
	fail "image project folder must be specified with -imageProjPath", :fatal
end
unless imageProjectFolder.pathExtension.isEqual_ "imageProj"
	fail "image project folder must have the path extension 'imageProj'", :fatal
end
unless NSFileManager.defaultManager.fileExistsAtPath(imageProjectFolder)
	fail "image project folder doesn't exist at specified path #{imageProjectFolder}", :fatal
end
imageProjectFolderLocation = NSURL.fileURLWithPath imageProjectFolder


# Utility Functions

def typeConformsToTypeInCollection(type, collection)
	collection.each do |currentCollectionType|
		next unless NSWorkspace.sharedWorkspace.type_conformsToType_ type, currentCollectionType
		return true
	end
	
	false
end

def eachLocationToType(locations)
	locations.each do |currentLocation|
		currentLocationUTType = currentLocation.getResourceValue_forKey_error NSURLTypeIdentifierKey
		unless currentLocationUTType[0]
			currentLocationUTTypeError = currentLocationUTType[2]
			fail "couldn't get resource type of item #{currentLocation}, #{currentLocationUTTypeError.localizedDescription} #{currentLocationUTTypeError.localizedRecoverySuggestion}"
			next
		end
		currentLocationUTType = currentLocationUTType[1]
		
		yield currentLocation, currentLocationUTType
	end
end


#

start

print "checking #{imageProjectFolder}\n"

#

# Check that all files are either plist or image files

supportedUTTypes = [ ]
supportedUTTypes << "public.image"
supportedUTTypes << "com.apple.property-list"

imageProjectContents = NSFileManager.defaultManager.contentsOfDirectoryAtURL_includingPropertiesForKeys_options_error imageProjectFolderLocation, NSArray.array, 0
unless imageProjectContents[0]
	imageProjectContentsError = imageProjectContents[1]
	fail "couldn't get contents of image project folder, #{imageProjectContentsError.localizedDescription} #{imageProjectContentsError.localizedRecoverySuggestion}", :fatal
end
imageProjectContents = imageProjectContents[0]

eachLocationToType imageProjectContents do |currentLocation, currentLocationUTType|
	unless typeConformsToTypeInCollection currentLocationUTType, supportedUTTypes
		fail "item type #{currentLocationUTType} doesn't conform to any of the supported image project folder contents UTTypes"
	end
end


# Collect all image location tuples

imageLocationTuples = []

totalImageLocations = 0

eachLocationToType imageProjectContents do |currentLocation, currentLocationUTType|
	skipCurrentLocation = false
	
	unless skipCurrentLocation
		skipCurrentLocation = NSWorkspace.sharedWorkspace.type_conformsToType_(currentLocationUTType, "public.image") != true
	end
	unless skipCurrentLocation
		# WARNING: add ICNS and TIFF checking
		excludedTypes = [ "com.apple.icns", "public.tiff" ]
		excludedTypes.each do |currentType|
			unless NSWorkspace.sharedWorkspace.type_conformsToType_ currentLocationUTType, currentType
				next
			end
			
			skipCurrentLocation = true
			break
		end
	end
	next if skipCurrentLocation
	totalImageLocations += 1
	
	next if /@\dx/m.match currentLocation.lastPathComponent
	
	currentTuple = [ currentLocation ]
	
	i = 2
	while true
		expectedCounterpartFilename = currentLocation.lastPathComponent.stringByDeletingPathExtension
		expectedCounterpartFileExtension = currentLocation.pathExtension
		
		currentContainerDirectory = currentLocation.URLByDeletingLastPathComponent
		expectedCounterpartLocation = currentContainerDirectory.URLByAppendingPathComponent("#{expectedCounterpartFilename}@#{i}x").URLByAppendingPathExtension(expectedCounterpartFileExtension)
		
		break unless NSFileManager.defaultManager.fileExistsAtPath expectedCounterpartLocation.path
		
		currentTuple << expectedCounterpartLocation
		i += 1
	end
	
	imageLocationTuples << currentTuple
end

print "-> found #{totalImageLocations} image files\n\n"


# Check that all images have at least @2x variants

print "checking that all @1x images have @2x counterparts\n"

imageLocationTuplesMissing2x = []

imageLocationTuples.each do |currentTuple|
	contains2x = false
	currentTuple.each do |currentLocation|
		contains2x = /@2x/.match currentLocation.lastPathComponent
		break if contains2x
	end
	next if contains2x
	
	imageLocationTuplesMissing2x << currentTuple
end

if imageLocationTuplesMissing2x.count == 0
	print "-> all normal resolution images have @2x counterparts\n"
else
	fail "#{imageLocationTuplesMissing2x.count} @1x resolution images are missing @2x counterparts"
	
	filenames = imageLocationTuplesMissing2x.to_ns.valueForKeyPath "@unionOfArrays.lastPathComponent"
	filenames.each do |currentFilename|
		puts "  #{currentFilename}"
	end
end
print "\n"


# Check that all images with @2x variants are actually exactly 2x the pixel resolution of their 1x versions

print "check that all @\\dx counterparts are exact pixel size multiples of their smallest representations\n"

imagesLocationTuplesWhoseImagesArentAllMultiplesOfTheBaseResolution = []

imageLocationTuples.each do |currentTuple|
	
	# Non multi tuples don't need to be checked
	
	next unless currentTuple.count > 1
	
	# Find the smallest size in the tuple
	
	currentSmallestValue = NSNotFound
	currentSmallestLocation = nil
	
	currentTuple.each do |currentLocation|
		currentMatch = /@(\d)x/.match(currentLocation.lastPathComponent)
		next if currentMatch.nil?
		
		currentValue = currentMatch[1]
		next unless currentValue.to_i < currentSmallestValue
		
		currentSmallestValue = currentValue
		currentSmallestLocation = currentLocation
	end
	next if currentSmallestLocation.nil?
	
	smallestImage = NSImage.alloc.initWithContentsOfURL currentSmallestLocation
	
	baseSize = smallestImage.size
	baseRatio = baseSize.height / baseSize.width
	
	# Ensure all images in the tuple are multiples of the base size
	
	currentTuple.each do |currentLocation|
		currentImage = NSImage.alloc.initWithContentsOfURL currentLocation
		currentSize = currentImage.size
		
		isMultiple = true
		isMultiple = isMultiple && ((currentSize.width % baseSize.width) == 0)
		isMultiple = isMultiple && ((currentSize.height % baseSize.height) == 0)
		
		ratioEqual = (currentSize.height / currentSize.width) == baseRatio
		
		next if isMultiple && ratioEqual
		
		imagesLocationTuplesWhoseImagesArentAllMultiplesOfTheBaseResolution << currentTuple
		break
	end
end

if imagesLocationTuplesWhoseImagesArentAllMultiplesOfTheBaseResolution.count == 0
	print "-> all @\\dx counterparts are exact pixel size multiples of their smallest representations\n"
else
	fail "some images aren't exact pixel size multiples of their smallest representation"
	
	filenames = imagesLocationTuplesWhoseImagesArentAllMultiplesOfTheBaseResolution.to_ns.valueForKeyPath "@unionOfArrays.lastPathComponent"
	filenames.each do |currentFilename|
		puts "  #{currentFilename}"
	end
end


# Check that TIFF images include 1x and 2x representationss

# unimplemented


#

finish
